"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const os_1 = __importDefault(require("os"));
const util_1 = __importDefault(require("util"));
const fsExistsAsync = util_1.default.promisify(fs_1.default.exists);
const fsReadFileAsync = util_1.default.promisify(fs_1.default.readFile);
function getDefaultSearchPlaces(name) {
    return [
        'package.json',
        `.${name}rc.json`,
        `.${name}rc.js`,
        `${name}.config.js`,
    ];
}
function getSearchPaths(startDir, stopDir) {
    return startDir
        .split(path_1.default.sep)
        .reduceRight((acc, _, ind, arr) => {
        const currentPath = arr.slice(0, ind + 1).join(path_1.default.sep);
        if (!acc.passedStopDir)
            acc.searchPlaces.push(currentPath);
        if (currentPath === stopDir)
            acc.passedStopDir = true;
        return acc;
    }, { searchPlaces: [], passedStopDir: false }).searchPlaces;
}
exports.defaultLoaders = Object.freeze({
    '.js': filepath => {
        return require(filepath);
    },
    '.json': filepath => {
        return require(filepath);
    },
    noExt(_, content) {
        try {
            return JSON.parse(content);
        }
        catch (e) {
            return null;
        }
    },
});
function getOptions(name, options = {}) {
    return Object.assign(Object.assign({ stopDir: os_1.default.homedir(), searchPlaces: getDefaultSearchPlaces(name), ignoreEmptySearchPlaces: true, transform: (x) => x, packageProp: [name] }, options), { loaders: Object.assign(Object.assign({}, exports.defaultLoaders), options.loaders) });
}
function getPackageProp(props, obj) {
    const propsArr = typeof props === 'string' ? props.split('.') : props;
    return (propsArr.reduce(
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    (acc, prop) => (acc == null ? acc : acc[prop]), obj) || null);
}
function getSearchItems(searchPlaces, searchPaths) {
    return searchPaths.reduce((acc, searchPath) => {
        searchPlaces.forEach(fileName => acc.push({
            fileName,
            filePath: path_1.default.join(searchPath, fileName),
            loaderKey: path_1.default.extname(fileName) || 'noExt',
        }));
        return acc;
    }, []);
}
function lilconfig(name, options) {
    const { ignoreEmptySearchPlaces, loaders, packageProp, searchPlaces, stopDir, transform, } = getOptions(name, options);
    return {
        async search(searchFrom = process.cwd()) {
            const searchPaths = getSearchPaths(searchFrom, stopDir);
            const result = {
                config: null,
                path: '',
            };
            const searchItems = getSearchItems(searchPlaces, searchPaths);
            for (const { fileName, filePath, loaderKey } of searchItems) {
                const exists = await fsExistsAsync(filePath);
                if (!exists)
                    continue;
                const content = String(await fsReadFileAsync(filePath));
                const loader = loaders[loaderKey];
                // handle package.json
                if (fileName === 'package.json') {
                    try {
                        const pkg = loader(filePath, content);
                        const maybeConfig = getPackageProp(packageProp, pkg);
                        if (maybeConfig != null) {
                            result.config = maybeConfig;
                            result.path = filePath;
                            break;
                        }
                    }
                    catch (err) { }
                    continue;
                }
                // handle other type of configs
                const isEmpty = content.trim() === '';
                if (isEmpty && ignoreEmptySearchPlaces)
                    continue;
                try {
                    result.config = require(filePath);
                    result.path = filePath;
                    if (isEmpty)
                        result.isEmpty = isEmpty;
                }
                catch (err) {
                    result.config = null;
                    throw new Error(`lol kek ${err}`);
                }
                break;
            }
            // not found
            if (result.path === '' && result.config === null)
                return transform(null);
            return transform(result);
        },
        async load(filePath) {
            const { base, ext } = path_1.default.parse(filePath);
            const loaderKey = ext || 'noExt';
            const loader = loaders[loaderKey];
            const content = String(await fsReadFileAsync(filePath));
            if (base === 'package.json') {
                const pkg = await loader(filePath, content);
                return transform({
                    config: getPackageProp(packageProp, pkg),
                    path: filePath,
                });
            }
            const result = {
                config: null,
                path: filePath,
            };
            // handle other type of configs
            const isEmpty = content.trim() === '';
            if (isEmpty && ignoreEmptySearchPlaces)
                return transform(null);
            // cosmiconfig returns undefined for empty files
            result.config = isEmpty
                ? undefined
                : await loader(filePath, content);
            return transform(isEmpty ? Object.assign(Object.assign({}, result), { isEmpty, config: undefined }) : result);
        },
    };
}
exports.lilconfig = lilconfig;
function lilconfigSync(name, options) {
    const { ignoreEmptySearchPlaces, loaders, packageProp, searchPlaces, stopDir, transform, } = getOptions(name, options);
    return {
        search(searchFrom = process.cwd()) {
            const searchPaths = getSearchPaths(searchFrom, stopDir);
            const result = {
                config: null,
                path: '',
            };
            const searchItems = getSearchItems(searchPlaces, searchPaths);
            for (const { fileName, filePath, loaderKey } of searchItems) {
                const exists = fs_1.default.existsSync(filePath);
                if (!exists)
                    continue;
                const loader = loaders[loaderKey];
                const content = String(fs_1.default.readFileSync(filePath));
                // handle package.json
                if (fileName === 'package.json') {
                    try {
                        const pkg = loader(filePath, content);
                        const maybeConfig = getPackageProp(packageProp, pkg);
                        if (maybeConfig != null) {
                            result.config = maybeConfig;
                            result.path = filePath;
                            break;
                        }
                    }
                    catch (err) { }
                    continue;
                }
                // handle other type of configs
                const isEmpty = content.trim() === '';
                if (isEmpty && ignoreEmptySearchPlaces)
                    continue;
                try {
                    result.config = loader(filePath, content);
                    result.path = filePath;
                    if (isEmpty)
                        result.isEmpty = isEmpty;
                }
                catch (err) {
                    result.config = null;
                }
                break;
            }
            // not found
            if (result.path === '' && result.config === null)
                return transform(null);
            return transform(result);
        },
        load(filePath) {
            const { base, ext } = path_1.default.parse(filePath);
            const loaderKey = ext || 'noExt';
            const loader = loaders[loaderKey];
            const content = String(fs_1.default.readFileSync(filePath));
            if (base === 'package.json') {
                const pkg = loader(filePath, content);
                return transform({
                    config: getPackageProp(packageProp, pkg),
                    path: filePath,
                });
            }
            const result = {
                config: null,
                path: filePath,
            };
            // handle other type of configs
            const isEmpty = content.trim() === '';
            if (isEmpty && ignoreEmptySearchPlaces)
                return transform(null);
            // cosmiconfig returns undefined for empty files
            result.config = isEmpty ? undefined : loader(filePath, content);
            return transform(isEmpty ? Object.assign(Object.assign({}, result), { isEmpty, config: undefined }) : result);
        },
    };
}
exports.lilconfigSync = lilconfigSync;
